import * as http from 'http';
import { IncomingHttpHeaders } from 'http';
import { HighlightOptions } from 'highlight.run';

interface NodeOptions extends HighlightOptions {
    /**
     * ID used to associate payloads with a Highlight project.
     */
    projectID: string;
    /**
     * The endpoint string to send opentelemetry data to.
     * @default https://otel.highlight.io:4318
     */
    otlpEndpoint?: string;
}

declare const HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
interface HighlightInterface {
    init: (options: NodeOptions) => void;
    stop: () => Promise<void>;
    isInitialized: () => boolean;
    parseHeaders: (headers: IncomingHttpHeaders) => {
        secureSessionId: string;
        requestId: string;
    } | undefined;
    consumeError: (error: Error, secureSessionId?: string, requestId?: string) => void;
    recordMetric: (secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]) => void;
    flush: () => Promise<void>;
    log: (message: any, level: string, ...optionalParams: any[]) => void;
    _debug: (...data: any[]) => void;
}
declare const H: HighlightInterface;

declare class Highlight {
    readonly FLUSH_TIMEOUT = 10;
    _intervalFunction: ReturnType<typeof setInterval>;
    _projectID: string;
    _debug: boolean;
    private otel;
    private tracer;
    private processor;
    constructor(options: NodeOptions);
    stop(): Promise<void>;
    _log(...data: any[]): void;
    recordMetric(secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]): void;
    log(date: Date, msg: string, level: string, stack: object, secureSessionId?: string, requestId?: string): void;
    consumeCustomError(error: Error, secureSessionId: string | undefined, requestId: string | undefined): void;
    flush(): Promise<void>;
}

/** JSDoc */
interface MiddlewareError extends Error {
    status?: number | string;
    statusCode?: number | string;
    status_code?: number | string;
    output?: {
        statusCode?: number | string;
    };
}
/**
 * Express compatible error handler.
 * Exposed as `Handlers.errorHandler`
 */
declare function errorHandler(options: NodeOptions): (error: MiddlewareError, req: http.IncomingMessage, res: http.ServerResponse, next: (error: MiddlewareError) => void) => void;
/**
 * A TRPC compatible error handler for logging errors to Highlight.
 */
declare function trpcOnError({ error, req, }: {
    error: Error;
    req: {
        headers?: http.IncomingHttpHeaders;
    };
}, options: NodeOptions): Promise<void>;
/**
 * A wrapper for logging errors to Highlight for Firebase HTTP functions
 */
declare type FirebaseHttpFunctionHandler = (req: any, resp: any) => void | Promise<void>;
declare function firebaseHttpFunctionHandler(origHandler: FirebaseHttpFunctionHandler, options: NodeOptions): FirebaseHttpFunctionHandler;
/**
 * A wrapper for logging errors to Highlight for Firebase callable functions
 */
declare type FirebaseCallableFunctionHandler = (data: any, context: any) => any;
declare function firebaseCallableFunctionHandler(origHandler: FirebaseCallableFunctionHandler, options: NodeOptions): FirebaseCallableFunctionHandler;
/**
 * A wrapper for logging errors to Highlight for AWS Lambda and other serverless functions
 */
declare type ServerlessCallableFunctionHandler = (event?: any) => any;
declare function serverlessFunction(origHandler: ServerlessCallableFunctionHandler, options: NodeOptions): ServerlessCallableFunctionHandler;

declare const handlers_errorHandler: typeof errorHandler;
declare const handlers_trpcOnError: typeof trpcOnError;
declare const handlers_firebaseHttpFunctionHandler: typeof firebaseHttpFunctionHandler;
declare const handlers_firebaseCallableFunctionHandler: typeof firebaseCallableFunctionHandler;
declare const handlers_serverlessFunction: typeof serverlessFunction;
declare namespace handlers {
  export {
    handlers_errorHandler as errorHandler,
    handlers_trpcOnError as trpcOnError,
    handlers_firebaseHttpFunctionHandler as firebaseHttpFunctionHandler,
    handlers_firebaseCallableFunctionHandler as firebaseCallableFunctionHandler,
    handlers_serverlessFunction as serverlessFunction,
  };
}

export { H, HIGHLIGHT_REQUEST_HEADER, handlers as Handlers, Highlight, NodeOptions };
